# Generated by Django 2.2.6 on 2020-02-25 09:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0013_review_author_anonym'),
    ]

    operations = [
        migrations.AddField(
            model_name='review',
            name='is_reviewd',
            field=models.BooleanField(default=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-08-02 11:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import sorl.thumbnail.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Screenshot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('target_id', models.PositiveIntegerField()),
                ('_target', models.BooleanField(default=True)),
                ('src', sorl.thumbnail.fields.ImageField(height_field='width', upload_to='screenshots/', width_field='width')),
                ('width', models.IntegerField()),
                ('height', models.IntegerField()),
                ('description', models.TextField(blank=True, null=True)),
                ('_id', models.CharField(blank=True, max_length=24, null=True)),
                ('target_content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Скриншот',
                'verbose_name_plural': 'Скриншоты',
            },
        ),
    ]



from .models import Screenshot

from django.contrib.contenttypes.fields import GenericRelation
from gfk_models.gfk_base.mixins import GFKItemsManager, GFKBaseMixin
from django.core.exceptions import ObjectDoesNotExist


class ScreenshotManager(GFKItemsManager):
    item_model = Screenshot


class ScreenshotMixin(GFKBaseMixin):
    screenshots = GenericRelation(Screenshot,
                            related_query_name='target',
                            object_id_field='target_id',
                            content_type_field='target_content_type')

    def questions_manager(self):
        return ScreenshotManager(self)

    class Meta:
        abstract = True



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models import Avg

from gfk_models.vote.models import VoteStatsAbstract, VoteAbstract


class ReviewStats(VoteStatsAbstract):
    # Average for connected vote objects
    avg = models.FloatField(default=0)

    recount_attr_list = ['count', 'sum', 'avg']

    def recount_avg(self):
        avg = self.get_recount_queryset().aggregate(Avg('rating'))['rating__avg']
        return round(avg, 1)


class Review(VoteAbstract):
    ACCEPTED_VALUES = [1, 2, 3, 4, 5]

    # target_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    # target_id = models.PositiveIntegerField()
    # target = GenericForeignKey('target_content_type', 'target_id')

    # review_list = models.ForeignKey(ReviewList, blank=True, default=None, null=True)
    # vote_list_attr_name = 'review_list'

    pros = models.TextField(null=True, blank=True)
    cons = models.TextField(null=True, blank=True)
    summary = models.TextField()
    advise = models.BooleanField()

    class Meta:
        verbose_name = 'Отзыв'
        verbose_name_plural = 'Отзывы'
        unique_together = ['target_content_type', 'target_id', 'author']

# Register signals to recount
ReviewStats.setup_relation(Review)


# class Review(models.Model):
#     target_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
#     target_id = models.PositiveIntegerField()
#     target = GenericForeignKey('target_content_type', 'target_id')
#
#     _id = models.CharField(max_length=24, unique=True, null=True, blank=True)
#
#     author = models.ForeignKey(User)
#
#     pros = models.TextField(null=True, blank=True)
#     cons = models.TextField(null=True, blank=True)
#     summary = models.TextField()
#
#     rating = models.IntegerField()
#     advise = models.BooleanField()
#
#     last_update = models.DateField(auto_now=True, null=True)
#
#     class Meta:
#         verbose_name = 'Отзыв'
#         verbose_name_plural = 'Отзывы'
#         unique_together = ('author', 'target_content_type', 'target_id')
#
#     @staticmethod
#     def filter_by_target(target):
#         return Review.objects.filter(target_id=target.id, target_content_type=ContentType.objects.get_for_model(target).id)
#
#     def save(self, *args, **kwargs):
#         super(Review, self).save(*args, **kwargs)
#
#         target = self.target_content_type.model_class().objects.get(id=self.target_id)
#
#         rating = Review.objects\
#                             .filter(target_content_type=self.target_content_type, target_id=self.target_id)\
#                             .aggregate(Avg('rating'))
#         # print(rating)
#         target.rating = round(rating['rating__avg'], 1)
#         target.save()
#
#     def __str__(self):
#         return "%s - %s" % (self.author, str(self.rating))




# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-25 08:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('qna', '0003_question__target'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='slug',
            field=models.CharField(blank=True, max_length=200, null=True, unique=True),
        ),
    ]



from django import template
from django.contrib.contenttypes.models import ContentType

register = template.Library()


@register.filter
def get_item_credentials(item):
    item_id = str(item.id)
    item_ct = str(ContentType.objects.get_for_model(item).id)
    return '%s-%s' % (item_id, item_ct)



from django.contrib import admin
from .models import Review


class ReviewAdmin(admin.ModelAdmin):
    exclude = ['last_update']
    list_display = ['author', 'rating','last_update']
    model = Review

admin.site.register(Review, ReviewAdmin)


# class ReviewListAdmin(admin.ModelAdmin):
#     model = ReviewList

# admin.site.register(ReviewList, ReviewListAdmin)



from .models import Question
from .forms import QuestionForm

from django.contrib.contenttypes.fields import GenericRelation
from gfk_models.gfk_base.mixins import GFKItemsManager, GFKBaseMixin
from django.core.exceptions import ObjectDoesNotExist


class QuestionItemsManager(GFKItemsManager):
    item_model = Question
    item_form = QuestionForm
    create_or_update_fields = ['author']


class QuestionsMixin(GFKBaseMixin):
    questions = GenericRelation(Question,
                            related_query_name='target',
                            object_id_field='target_id',
                            content_type_field='target_content_type')

    def questions_manager(self):
        return QuestionItemsManager(self)

    class Meta:
        abstract = True



# Generated by Django 2.2.6 on 2020-02-25 13:43

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0015_auto_20200225_1301'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='author_anonym',
            field=models.CharField(default='Аноним', max_length=30, validators=[django.core.validators.MaxLengthValidator(30)]),
        ),
        migrations.AlterField(
            model_name='review',
            name='is_reviewd',
            field=models.BooleanField(default=False),
        ),
    ]



from django.views.generic import list, detail
from django.views import generic
from django.contrib.contenttypes.models import ContentType
from helpers.functions.common import CommonHelpers

class GFKBaseViewMixin(object):
    target_model = None
    target_request_attr = None
    target_retrieve_attr = None

    def get_target_from_dict(self, dict):
        # print(dict)
        if 'target_id' in dict and dict['target_id'] != "" \
            and 'target_content_type' in dict and dict['target_content_type'] != "":
            self.target = CommonHelpers.get_target(dict['target_content_type'], dict['target_id'])
            return True
        return False

    def get_target(self):
        '''
        If

        target_model
        target_request_attr
        target_retrieve_attr

        we looking for them in kwargs

        because we can attache views in such way:

        url(r'^foo/(?P<foo_slug>[\w\-]+)/qna/(?P<bar_id>[\d]+)-(?P<bar_slug>[\w\-]+)/',
                include('bar.urls'),
                {
                    'target_model': models.Question
                    'target_request_attr': 'foo_slug'
                    'target_retrieve_attr':'slug'
                }
            ),

        :return: Target connected thru GenericForeignKey
        '''
        try:
            target_model = self.get__kwargs['target_model']
        except:
            target_model = self.target_model

        try:
            target_request_attr = self.get__kwargs['target_request_attr']
        except:
            target_request_attr = self.target_request_attr

        try:
            target_retrieve_attr = self.get__kwargs['target_retrieve_attr']
        except:
            target_retrieve_attr = self.target_retrieve_attr

        try:
            kwargs = {}
            kwargs[target_retrieve_attr] = self.get__kwargs[target_request_attr]

            self.target = target_model.objects.get(*[], **kwargs)

            return self.target
        except:
            return None

    def dispatch(self, request, *args, **kwargs):
        self.populate_self(request, *args, **kwargs)
        return super(GFKBaseViewMixin, self).dispatch(request, *args, **kwargs)

    def get_queryset(self):
        '''

        self.get__kwargs - должен быть добавлен в дочернем классе
                * target_content_type
                * target_id

        :return:
        '''
        queryset = self.model.objects.all()

        if hasattr(self, 'target'):
            queryset = queryset.filter(_target=self.target)

        self.queryset = queryset

        return super(GFKBaseViewMixin, self).get_queryset()

    def populate_context(self, **kwargs):
        if hasattr(self, 'target'):
            kwargs['target'] = self.target

        if hasattr(self, 'parent_urls_namespace'):
            kwargs['parent_urls_namespace'] = self.parent_urls_namespace

        return kwargs

    def get_context_data(self, **kwargs):
        kwargs = self.populate_context(**kwargs)
        return super(GFKBaseViewMixin, self).get_context_data(**kwargs)

    def populate_self(self, request, *args, **kwargs):
        self.get__kwargs = kwargs
        self.get__args = args

        if not hasattr(self, 'target'):
            self.get_target_from_dict(request.GET)

        if not hasattr(self, 'target'):
            self.get_target_from_dict(request.POST)

        if not hasattr(self, 'target'):
            self.get_target()

        if 'parent_urls_namespace' in kwargs:
            self.parent_urls_namespace = kwargs['parent_urls_namespace']

        if hasattr(self, 'target'):
            kwargs['target'] = self.target
            kwargs['target_id'] = self.target.id
            kwargs['target_content_type'] = ContentType.objects.get_for_model(self.target).id

        post_dict = request.POST.copy()
        post_dict.update(kwargs)
        request.POST = post_dict


class GFKListView(GFKBaseViewMixin, list.ListView):
    pass


class GFKDetailView(GFKBaseViewMixin, detail.DetailView):
    pass


class GFKCreateView(GFKBaseViewMixin, generic.CreateView):
    pass


class GFKUpdateView(GFKBaseViewMixin, generic.CreateView):
    pass






from django.conf.urls import url, include
from . import views, models

urlpatterns = [
    url(r'^questions$', views.QuestionListView.as_view(), name='list'),

    url(r'^question/(?P<q_id>[\d]+)/edit$',
        views.QuestionUpdateView.as_view(), name='question_edit'),

    url(r'^question/(?P<q_id>[\d]+)/answer/add$',
        views.AnswerCreateView.as_view(), name='answer_add'),

    url(r'^answer/(?P<a_id>[\d]+)/edit$',
        views.AnswerUpdateView.as_view(), name='answer_edit'),

    url(r'^question/(?P<q_id>[\d]+)/answer/(?P<a_id>[\d]+)/accept$',
        views.QuestionAcceptAnswerView.as_view(), name='answer_accept'),


    # Последние чтобы слаг не перебивал случайно
    url(r'^question/(?P<q_id>[\d]+)/(?P<q_slug>[\w\-]+)',
        views.QuestionDetailView.as_view(), name='question_single'),

    url(r'^qna/add', views.QuestionCreateView.as_view(), name='add'),
]




# Generated by Django 2.2.6 on 2020-02-25 09:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0012_auto_20200225_0905'),
    ]

    operations = [
        migrations.AddField(
            model_name='review',
            name='author_anonym',
            field=models.CharField(blank=True, default='Аноним', max_length=100, null=True),
        ),
    ]



from django.contrib.auth.models import User
from django.urls import reverse
from django.db import models

from gfk_models.gfk_base.models import GFKModel
from gfk_models.gfk_base.models import ContentType
from gfk_models.like.mixins import LikesMixin
from gfk_models.gfk_helpers.functions.common import CommonHelpers
import datetime
from hitcount.models import HitCountMixin


class Question(LikesMixin, HitCountMixin, GFKModel):
    ITEMS_PER_PAGE = 30

    target_content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, null=True, blank=True)
    target_id = models.PositiveIntegerField(null=True, blank=True)

    author = models.ForeignKey(User, on_delete=models.CASCADE)
    title = models.CharField(max_length=200, unique=True, null=True, blank=True)
    slug = models.CharField(max_length=200, unique=True, null=True, blank=True)
    text = models.TextField()

    has_answer = models.BooleanField(default=False)

    last_update = models.DateTimeField(auto_now=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, null=True)

    class Meta:
        verbose_name = 'Вопрос'
        verbose_name_plural = 'Вопросы'
        ordering = ['-created_at']

    def __str__(self):
        return self.title

    def get_slug(self):
        if self.slug is None or self.slug == '':
            self.slug = CommonHelpers.get_slug(self.title)
            self.save()

        return self.slug

    def get_absolute_url(self):
        url = reverse('qna:question_single', kwargs={
            'q_id': self.id,
            'q_slug': self.get_slug(),
        })
        return url

    def get_edit_url(self):
        return reverse('qna:question_edit', args=[self.id])

    def toggle_accepted_answer(self, answer_id, user):
        if user.id != self.author.id:
            return False

        # print(answer_id, user, self.author)

        try:
            answer = Answer.objects.get(question=self.id, id=answer_id)

            Answer.objects.filter(question=self.id).exclude(id=answer.id).update(accepted=False)

            answer.accepted = not answer.accepted
            answer.save()

            self.has_answer = answer.accepted
            self.save()

            return answer.accepted
        except Exception as ex:
            # print(ex)
            return False

    @classmethod
    def get_popular_today(cls):
        yesterday = datetime.date.today() - datetime.timedelta(days=1)
        objs = cls.objects.filter(created_at__gt=yesterday)\
                        .order_by('-likes_stats__sum')[:10]

        if not objs:
            objs = cls.objects.filter().order_by('-likes_stats__sum')[:10]

        return objs


class Answer(LikesMixin, models.Model):
    question = models.ForeignKey(Question, on_delete=models.CASCADE)

    author = models.ForeignKey(User, on_delete=models.CASCADE)
    text = models.TextField()
    accepted = models.BooleanField(default=False)

    last_update = models.DateField(auto_now=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True, null=True)

    def get_absolute_url(self):
        url = self.question.get_absolute_url()
        return "%s#a%s" % (url, self.id)

    def get_edit_url(self):
        return reverse('qna:answer_edit', kwargs={'a_id':self.id})

    class Meta:
        verbose_name = 'Ответ'
        verbose_name_plural = 'Ответы'
        unique_together = ('author', 'question')
        ordering = ['-accepted', '-last_update']

    def __str__(self):
        return "Ответ - %s" % str(self.author)




# -*- coding: utf-8 -*-
# Generated by Django 1.9.3 on 2016-04-13 05:17
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0002_auto_20160412_1603'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='review',
            unique_together=set([('author', 'target_content_type', 'target_id')]),
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from django.forms.forms import ValidationError
from django.utils.translation import ugettext as _
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models


class GFKBaseManager(object):
    item_model = None
    # Item model form - used for adding new Item's to ItemList
    item_form = None

    base_create_or_update_fields = ['target_id', 'target_content_type']
    create_or_update_fields = []

    def set_kwargs(self, kwargs):
        return kwargs

    # def all(self, *args, **kwargs):
    #     self.set_kwargs(kwargs)
    #     return self.item_model.objects.filter(*args, **kwargs)
    #
    # def get(self, *args, **kwargs):
    #     self.set_kwargs(kwargs)
    #     return self.item_model.objects.get(*args, **kwargs)
    #
    # def filter(self, *args, **kwargs):
    #     self.set_kwargs(kwargs)
    #     return self.item_model.objects.filter(*args, **kwargs)

    def get__create_or_update_fields(self):
        return self.create_or_update_fields + self.base_create_or_update_fields

    def get_queryset__by_create_or_update_fields(self, data):
        '''
        Build filter to retrieve Item instance
            * base_create_or_update_fields
            * create_or_update_fields

            * - reuired fields
        :param data:
        :return: queryset.get
        '''
        kwargs = {}
        for field in self.get__create_or_update_fields():
            kwargs[field] = data[field]

        return self.item_model.objects.get(**kwargs)

    def add(self, review_dict):
        self.set_kwargs(review_dict)

        # try:
        #     item = self.get_queryset__by_create_or_update_fields(review_dict)
        # except ObjectDoesNotExist:
        item = self.item_model()

        form = self.item_form(review_dict, instance=item)

        if form.is_valid():
            return form.save()
        else:
            raise ValidationError(_('Form is invalid'), form)


class GFKItemsManager(GFKBaseManager):
    '''
    Use for GFKModel's appended directly to target
    '''

    def __init__(self, target):
        self.target = target

    def set_kwargs(self, kwargs):
        kwargs['target_id'] = self.target.id
        kwargs['target_content_type'] = self.item_model.get_content_type(self.target)
        return kwargs


class GFKItemsStatsManager(GFKItemsManager):
    # stats model
    stats_model = None
    '''
    Use for GFKModel's Items appended to targets thru lists (f.e. likes or reviews)
    '''
    def stats(self):
        try:
            stats = self.stats_model.objects.get(_target=self.target)
        except ObjectDoesNotExist:
            stats = self.stats_model(target=self.target)
            stats.save()

        return stats


class GFKBaseMixin(models.Model):
    class Meta:
        abstract = True
    # def __init__(self, relation_name, relation_model, manager_model, *args, **kwargs):
    #     '''
    #     :param relation_name: name of relation for target model
    #     :param args:
    #     :param kwargs:
    #     '''
    #
    #     print('GFKBaseMixin __init__ ->')
    #
    #     self.__relation_name = relation_name
    #     self.__relation_model = relation_model
    #     self.__manager_model = manager_model
    #
    #     setattr(self, relation_name, GenericRelation(relation_model))
    #     setattr(self, relation_name + "_manager", self.__get_manager)
    #
    #     super(GFKBaseMixin, self).__init__(*args, **kwargs)
    #
    # def __get_manager(self):
    #     return self.__manager_model(self)


# class GFKBaseStatsMixin(GFKBaseMixin):
#     def __init__(self, *args, **kwargs):
#         '''
#         :param relation_name: name of relation for target model
#         :param args:
#         :param kwargs:
#         '''
#         super(GFKBaseMixin, self).__init__(*args, **kwargs)



from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from django.http.response import JsonResponse
from django.utils.decorators import method_decorator

from gfk_models.gfk_base.models import GFKModel
from gfk_models.gfk_base.views import GFKCreateView
from gfk_models.gfk_helpers.functions.common import CommonHelpers


class LikeCreateView(GFKCreateView):
    @method_decorator(csrf_exempt)
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        kwargs['target_model'] = GFKModel.get_ct_model_static(kwargs['target_ct'])
        kwargs['target_request_attr'] = 'target_id'
        kwargs['target_retrieve_attr'] = 'id'

        return super(LikeCreateView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        author = CommonHelpers.get_user__by_request(request)

        if request.POST.get('v') == 'like':
            response = self.target.like(author)

        elif request.POST.get('v') == 'dislike':
            response = self.target.dislike(author)

        return JsonResponse({
            'likes': {
                'count': response.count,
                'sum': response.sum,
            }
        })



# Generated by Django 2.2.6 on 2019-11-01 11:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('review', '0010_remove_review__id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='_target',
            field=models.BooleanField(blank=True, default=True),
        ),
        migrations.AlterField(
            model_name='reviewstats',
            name='_target',
            field=models.BooleanField(blank=True, default=True),
        ),
    ]



# Generated by Django 2.2.6 on 2020-02-25 09:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('like', '0002_auto_20191101_1112'),
    ]

    operations = [
        migrations.AlterField(
            model_name='like',
            name='author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]
